package it.mokabyte.pattern.ejb.session.stateless;

import java.util.*;
import java.text.*;

import javax.ejb.*;
import javax.mail.*;
import javax.activation.*;
import javax.mail.internet.*;
import java.rmi.RemoteException;
import javax.naming.*;

import it.mokabyte.pattern.exception.MailException;
import it.mokabyte.pattern.valueobject.MailOM;
/**
 * <p>Title: MailerBean</p>
 * <p>Description: invia una mail mediante le javax.mail API </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author S: Rossini
 * @version 1.0
 */
public class MailerBean implements SessionBean {

  SessionContext sc;

  private static final String sender = "J2EE DEMO: Admin Mailer";

  // Costructor
  public MailerBean() {}

  // Life cycle methods
  public void ejbCreate() { System.out.println("MailerBeanSL : ejbCreate ..."); }
  public void ejbRemove() {}
  public void ejbActivate() {}
  public void ejbPassivate() {}
  public void setSessionContext(SessionContext sc) {}

  /**
    * Legge il value object MailOM ed esegue:
    * <li> Per i campi null - no set
    * <li> Per i campi valorizzati - set mail properties
    * A deploy time si definisce
    * <li> ejb.jar.xml:
    * <li><res-ref-name>mail/TheMailSession</res-ref-name>
    * <li><res-type>javax.mail.Session</res-type>
    * <li> vendor.xml
    * <li> <res-ref-name>mail/TheMailSession</res-ref-name>
    * <li> BAS <jndi-name>mail-borland://&mail.from=srossini@imolinfo.it&mail.host=imolinfo.it</jndi-name>
    * <li> WLS <jndi-name>TheMailSession</jndi-name>
    * <br> dove <b>mail.from</b> and <b>mail.host</b> sono configurate a deploy time
    */
  public void sendNotice(MailOM posta) throws MailException {
    try {
      System.out.println("MailerBeanSL.sendNotice: posta["+posta+"]...");
      // Setup mail server
      if(posta.getHostMailServer() != null){
        System.setProperty("mail.smtp.host", posta.getHostMailServer());
      }

      System.out.println("MailerBeanSL.sendNotice: HOST MAIL[" + System.getProperty("mail.smtp.host")+"]...");
      Context initial = new InitialContext();
      System.out.println("MailerBeanSL.sendNotice: BEFORE LOOKUP [java:comp/env/mail/TheMailSession]...");

      javax.mail.Session session =(javax.mail.Session) initial.lookup("java:comp/env/mail/TheMailSession");
      System.out.println("MailerBeanSL.sendNotice: AFTER LOOKUP [java:comp/env/mail/TheMailSession]...");
      System.out.println("MailerBeanSL.sendNotice: Mail session found [" + session + "]...");
      javax.mail.Message msg = new MimeMessage(session);
      if(posta.getFrom() == null){
          msg.setFrom();
      }
      else{
          msg.setFrom(new InternetAddress(posta.getFrom()));
      }

      String recipient = posta.getRecipient();
      System.out.println("MailerBeanSL.sendNotice: Recipient ["+recipient+"]...");
      msg.setRecipients(Message.RecipientType.TO,InternetAddress.parse(recipient, false));

      // SUBJECT of mail
      if(posta.getSubject() == null){
          msg.setSubject("Message from MailerEJB");
      }
      else{
          msg.setSubject(posta.getSubject());
      }
      DateFormat dateFormatter = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.SHORT);

      Date timeStamp = new Date();

      // MESSAGE BODY
      if(posta.getBodyMail() != null){
          msg.setText(posta.getBodyMail()+"-" + dateFormatter.format(timeStamp) + ".");
      }
      else{
          String messageText = "Thank you for your order." + '\n' +
             "We received your order on " +
             dateFormatter.format(timeStamp) + ".";
             msg.setText(messageText);
      }

      System.out.println("MailerBeanSL.sendNotice : MSG [" + msg +"]... ");

      msg.setHeader("X-Mailer", sender);
      msg.setSentDate(timeStamp);

      System.out.println("MailerBeanSL.sendNotice: BEFORE Transport.send of["+msg+"]...");
      javax.mail.Transport.send(msg);
      System.out.println("MailerBeanSL.sendNotice: AFTER Transport.send- Mail SENT!");

    } catch(Exception e) {
      System.out.println(" # Exception ! MailerBeanSL.sendNotice : " + e.getMessage());
      e.printStackTrace();
      throw new MailException(e);
    }
  }

}


