package it.mokabyte.pattern.ejb.messagedriven;

import java.io.*;
import java.io.Serializable;
import java.rmi.RemoteException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.CreateException;

import javax.jms.*;
import javax.naming.*;

import it.mokabyte.pattern.ejb.session.stateless.FacadeHome;
import it.mokabyte.pattern.ejb.session.stateless.Facade;
import it.mokabyte.pattern.locator.MyServiceLocator;
import it.mokabyte.pattern.util.JndiConfig;
import it.mokabyte.pattern.valueobject.MailOM;
import it.mokabyte.pattern.exception.MailException;
import it.mokabyte.pattern.ejb.session.stateless.MailerHome;
import it.mokabyte.pattern.ejb.session.stateless.Mailer;

/**
 * The MessageBean class is a message-driven bean.
 * Implementa le interfacce javax.ejb.MessageDrivenBean e javax.jms.MessageListener
 * Riceve i messaggi dalla coda e li processa
 */
public class MessageFacadeBean implements MessageDrivenBean, MessageListener {

  private transient MessageDrivenContext mdc = null;
  private Context context;

  /**
    * Constructor, which is public and takes no arguments.
    */
    public MessageFacadeBean() {
        System.out.println("MessageFacadeBean::MessageFacadeBean: ...");
    }

    /**
     * setMessageDrivenContext method, declared as public (but not
     * final or static), with a return type of void, and with one
     * argument of type javax.ejb.MessageDrivenContext.
     */
    public void setMessageDrivenContext(MessageDrivenContext mdc){
        System.out.println("MessageFacadeBean.setMessageDrivenContext() : mdc["+mdc.getClass().getName()+"]");
	  this.mdc = mdc;
    }

    /**
     * ejbCreate method, declared as public (but not final or
     * static), with a return type of void, and with no arguments.
     */
    public void ejbCreate() {
	System.out.println("MessageFacadeBean.ejbCreate()");
    }

    /**
     * onMessage method, declared as public (but not final or static)
     * return type of void - one argument javax.jms.Message
     * Casts the incoming Message to a TextMessage and process it
      * @param inMessage	the incoming message
     */
  public void onMessage(Message inMessage) {

    ObjectMessage objMsg = null;

    System.out.println("MessageFacadeBean.onMessage: inMessage["+inMessage+"]...");
    System.out.println("MessageFacadeBean.onMessage: inMessage class ["+inMessage.getClass().getName()+"]...");

    try {
      if (inMessage instanceof ObjectMessage) {
        objMsg = (ObjectMessage) inMessage;
        Object obj = objMsg.getObject();
        System.out.println("MessageFacadeBean.onMessage: Message RX["+obj+"]-class["+obj.getClass().getName()+"]...");
        if(obj instanceof MailOM) {

          // *************************** Ciclo di consumo CPU***********************
          // E' per didattica: serve per enfatizzare i vantaggi del pattern
          for(int i=0; i<10000; ++i){
            i = (i-1)+1;
            System.out.println("Ciclo di elaborazione dummy: " + i);
          }
          // ***********************************************************************

          MailOM theMail = (MailOM)objMsg.getObject();
          System.out.println("MessageFacadeBean.onMessage: creazione Mailer EJB ["+JndiConfig.ENV_MAILER+"]...");
          MailerHome home = (MailerHome)MyServiceLocator.getHome(JndiConfig.ENV_MAILER,MailerHome.class);
          System.out.println("MessageFacadeBean.onMessage: home["+home+"]-class["+home.getClass().getName()+"]...");
          Mailer mailer = home.create();
          System.out.println("MessageFacadeBean.onMessage: mailer["+mailer+"]-class["+mailer.getClass().getName()+"]...");
          mailer.sendNotice(theMail);
          System.out.println("MessageFacadeBean.onMessage: eseguito!");
        }
        else{
          System.out.println("MessageFacadeBean.onMessage: NESSUNA azione intrapresa per il contenuto del messaggio di classe["+obj.getClass().getName()+"]...");
        }
      }
      else {
        System.out.println("MessageFacadeBean.onMessage: Message JMS Ricevuto NON di classe ObjectMessage !");
      }
    } catch (JMSException jmse) {
        System.out.println(" # JMSEXCEPTION # MessageFacadeBean: " + jmse.getMessage());
        jmse.printStackTrace();
        // this.mdc.setRollbackOnly();
    } catch (Exception e) {
        System.out.println(" # EXCEPTION # MessageFacadeBean: " + e.getMessage());
        e.printStackTrace();
        // this.mdc.setRollbackOnly();
    }
  }

  /**
    * ejbRemove method, declared as public (but not final or
    * static), with a return type of void, and with no arguments.
    */
  public void ejbRemove() {
    System.out.println("MessageFacadeBean.remove: ...");
  }
}
