package it.mokabyte.pattern.client;

import javax.jms.*;
import javax.naming.*;
import java.util.Properties;

import it.mokabyte.pattern.ejb.session.stateless.FacadeHome;
import it.mokabyte.pattern.ejb.session.stateless.Facade;
import it.mokabyte.pattern.valueobject.MailOM;

import it.mokabyte.pattern.locator.MyServiceLocator;
import it.mokabyte.pattern.exception.ServiceLocationException;
import it.mokabyte.pattern.util.JndiConfig;

import it.mokabyte.pattern.util.Chronus;
/**
 * <p>Title: MessageFacadeTestClient</p>
 * <p>Description: client dell'EJB Message Facade
 * Il MDB  listener sulla coda "queue/MokaFacadeQueue"</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author S.Rossini
 * @version 1.0
 */

public class MessageFacadeTestClient {

  public MessageFacadeTestClient() {

    QueueConnection queueConnection = null;

    try {

      Context ctx = new InitialContext();
      QueueConnectionFactory queueConnectionFactory = (QueueConnectionFactory)ctx.lookup("MokaQueueConnectionFactory");
      // QueueConnectionFactory queueConnectionFactory = (QueueConnectionFactory)MyServiceLocator.getJmsConnectionFactory(JndiConfig.MOKA_CONNECTON_FACTORY);
      System.out.println("QueueConnectionFactory["+queueConnectionFactory+"]...");

      Queue queue = (Queue)ctx.lookup("queue/MokaFacadeQueue");
      // Queue queue = (Queue)MyServiceLocator.getJmsDestination(JndiConfig.MOKA_FACADE_QUEUE);
      System.out.println("Queue["+queue+"]...");

      queueConnection =queueConnectionFactory.createQueueConnection();
      System.out.println("QueueConnection["+queueConnection+"]...");

      QueueSession queueSession =queueConnection.createQueueSession(false,Session.AUTO_ACKNOWLEDGE);
      System.out.println("QueueSession["+queueSession+"]...");

      QueueSender queueSender = queueSession.createSender(queue);
      System.out.println("QueueSender["+queueSender+"]...");

      // si crea il messaggio
      ObjectMessage message = queueSession.createObjectMessage();

      MailOM lettera = new MailOM("srossini@mokabyte.it","CIAO!!!");
      System.out.println("Mail da inviare: " + lettera);

      message.setObject(lettera);

      Chronus krono = new Chronus();
      System.out.println(new java.util.Date().toString()+": prima dell'invio del messaggio...");
      krono.begin();
      // invio del messaggio
      queueSender.send(message);
      krono.end();
      System.out.println(new java.util.Date().toString()+": messaggio inviato!");
      System.out.println("Tempo trascorso(H:m:s):" + Chronus.convertInTime(krono.getElapsed()));
    } catch (NamingException e) {
        System.out.println(" # Exception: " +e.toString());
    } catch (JMSException e) {
        System.out.println(" # Exception: " +e.toString());
    } finally {
        if (queueConnection != null) {
          try {
            queueConnection.close();
            System.out.println("queueConnection chiusa.");
          } catch (JMSException e) {}
        }
    }
  }

  private Context getInitialContext() throws Exception {
    String url = "t3://localhost:7001";
    String user = null;
    String password = null;
    Properties properties = null;
    try {
      properties = new Properties();
      properties.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
      properties.put(Context.PROVIDER_URL, url);
      if (user != null) {
        properties.put(Context.SECURITY_PRINCIPAL, user);
        properties.put(Context.SECURITY_CREDENTIALS, password == null ? "" : password);
      }
      return new InitialContext(properties);
    }
    catch(Exception e) {
      System.out.println("Unable to connect to WebLogic server at " + url);
      System.out.println("Please make sure that the server is running.");
      throw e;
    }
  }

  /**Main method*/
  public static void main(String[] args) {
      new MessageFacadeTestClient();
  }
}
